Imports System.Xml

Module XmlStuff

    ' Return this node's document root object.
    Private Function FindXmlDocument(ByVal xml_node As XmlNode) As XmlDocument
        Dim xml_document As XmlDocument

        xml_document = xml_node.OwnerDocument
        If xml_document Is Nothing Then xml_document = xml_node
        Return xml_document
    End Function

    ' Add an XML declaration to the document.
    Public Function AddXmlDeclaration( _
      ByVal xml_node As XmlNode, _
      Optional ByVal version As String = "1.0", _
      Optional ByVal encoding As String = Nothing, _
      Optional ByVal standalone As String = Nothing) _
      As XmlDeclaration

        ' Create the new XmlDeclaration.
        Dim xml_declaration As XmlDeclaration
        xml_declaration = FindXmlDocument(xml_node).CreateXmlDeclaration( _
            version, encoding, standalone)

        ' Prepend the child so we know it's in the right place.
        FindXmlDocument(xml_node).PrependChild(xml_declaration)
        Return xml_declaration
    End Function

    ' Create an XmlProcessingInstruction.
    Public Function AddXmlProcessingInstruction( _
      ByVal xml_node As XmlNode, _
      ByVal target As String, _
      ByVal data As String) _
      As XmlProcessingInstruction

        ' Create the new processing instruction.
        Dim xml_processing_instruction As XmlProcessingInstruction
        xml_processing_instruction = _
            FindXmlDocument(xml_node).CreateProcessingInstruction(target, data)
        xml_node.AppendChild(xml_processing_instruction)
        Return xml_processing_instruction
    End Function

    ' Associate the XML file with an XSL file.
    Public Function AddXmlXSLFile( _
      ByVal xml_node As XmlNode, _
      ByVal xsl_file As String) _
      As XmlProcessingInstruction

        ' Add the appropriate processing instruction to
        ' the document.
        Return AddXmlProcessingInstruction( _
            FindXmlDocument(xml_node), "xml-stylesheet", _
            "type=""text/xsl"" href=""" & xsl_file & """")
    End Function

    ' Create a DOCTYPE section.
    Public Function AddXmlDocumentType( _
      ByVal xml_node As XmlNode, _
      ByVal document_name As String, _
      ByVal internal_subset As String, _
      Optional ByVal public_id As String = Nothing, _
      Optional ByVal system_id As String = Nothing) _
      As XmlDocumentType

        ' Create the new DOCTYPE section.
        Dim xml_document_type As XmlDocumentType
        xml_document_type = FindXmlDocument(xml_node).CreateDocumentType( _
            document_name, public_id, system_id, _
            internal_subset)
        FindXmlDocument(xml_node).AppendChild(xml_document_type)
        Return xml_document_type
    End Function

    ' Add a new XmlElement to the node.
    Public Function AddXmlElement( _
      ByVal parent_node As XmlNode, _
      ByVal element_name As String) _
      As XmlElement

        ' Create the new element.
        Dim xml_element As XmlElement
        xml_element = FindXmlDocument(parent_node).CreateElement(element_name)
        parent_node.AppendChild(xml_element)
        Return xml_element
    End Function

    ' Add an XmlAttribute to the node.
    Public Function AddXmlAttribute( _
      ByVal xml_node As XmlNode, _
      ByVal attribute_name As String, _
      ByVal attribute_value As String) _
      As XmlAttribute

        ' Create the new attribute.
        Dim xml_attribute As XmlAttribute
        xml_attribute = FindXmlDocument(xml_node).CreateAttribute(attribute_name)
        xml_attribute.Value = attribute_value
        xml_node.Attributes.Append(xml_attribute)
        Return xml_attribute
    End Function

    ' Add an XmlWhitespace object containing a vbCrLf and the
    ' indicated number of spaces to the parent node.
    Public Function AddXmlNewLine( _
      ByVal parent_node As XmlNode, _
      ByVal num_spaces As Integer) _
      As XmlWhitespace

        ' Make the XmlWhitespace object.
        Dim xml_whitespace As XmlWhitespace
        xml_whitespace = FindXmlDocument(parent_node).CreateWhitespace( _
            vbCrLf & Space(num_spaces))
        parent_node.AppendChild(xml_whitespace)
        Return xml_whitespace
    End Function

    ' Add a comment to the node.
    Public Function AddXmlComment( _
      ByVal parent_node As XmlNode, _
      ByVal comment_value As String) _
      As XmlComment

        ' Make the comment.
        Dim xml_comment As XmlComment
        xml_comment = FindXmlDocument(parent_node).CreateComment(" " & comment_value & " ")
        parent_node.AppendChild(xml_comment)
        Return xml_comment
    End Function

    ' Add some CDATA to the node.
    Public Function AddXmlCdataSection( _
      ByVal parent_node As XmlNode, _
      ByVal cdata_text As String) _
      As XmlCDataSection

        ' Make the CDATA section.
        Dim xml_cdata_section As XmlCDataSection
        xml_cdata_section = FindXmlDocument(parent_node).CreateCDataSection(cdata_text)
        parent_node.AppendChild(xml_cdata_section)
        Return xml_cdata_section
    End Function

    ' Add a new XmlText node to the node.
    Public Function AddXmlText( _
      ByVal parent_node As XmlNode, _
      ByVal element_text As String) _
      As XmlText

        ' Make the XmlText node.
        Dim xml_text As XmlText
        xml_text = FindXmlDocument(parent_node).CreateTextNode(element_text)
        parent_node.AppendChild(xml_text)
        Return xml_text
    End Function

    ' Add an entitiy reference to the node.
    Public Function AddXmlEntityReference( _
      ByVal parent_node As XmlNode, _
      ByVal entity_name As String) _
      As XmlEntityReference

        ' Make the entity reference.
        Dim xml_entity_reference As XmlEntityReference
        xml_entity_reference = FindXmlDocument(parent_node).CreateEntityReference(entity_name)
        parent_node.AppendChild(xml_entity_reference)
        Return xml_entity_reference
    End Function

    ' Add an XmlWhitespace object.
    Public Function AddXmlWhitespace( _
      ByVal parent_node As XmlNode, _
      ByVal whitespace As String) _
      As XmlWhitespace

        ' Make the XmlWhitespace object.
        Dim xml_whitespace As XmlWhitespace
        xml_whitespace = FindXmlDocument(parent_node).CreateWhitespace(whitespace)
        parent_node.AppendChild(xml_whitespace)
        Return xml_whitespace
    End Function

    ' Add an XmlSignificantWhitespace object.
    Public Function AddXmlSignificantWhitespace( _
      ByVal parent_node As XmlNode, _
      ByVal whitespace As String) _
      As XmlSignificantWhitespace

        ' Make the XmlSignificantWhitespace object.
        Dim xml_whitespace As XmlSignificantWhitespace
        xml_whitespace = FindXmlDocument(parent_node).CreateSignificantWhitespace(whitespace)
        parent_node.AppendChild(xml_whitespace)
        Return xml_whitespace
    End Function

End Module
