Attribute VB_Name = "NewMacros"
' Load the indicated XML file containing an article
' into the currently active document.
Sub LoadXmlArticle(ByVal xml_file_name As String)
    Dim dom_document As DOMDocument
    Dim article_node As IXMLDOMNode
    Dim section_node As IXMLDOMNode
    Dim paragraph_node As IXMLDOMNode
    Dim section_title As String

    ' Load the XML file into a DOM representation.
    Set dom_document = New DOMDocument
    dom_document.Load xml_file_name

    ' Process the Article node.
    Set article_node = dom_document.documentElement
    Selection.Style = ActiveDocument.Styles.Item("Heading 1")
    Selection.TypeText Trim$(article_node.Attributes(0).nodeValue)
    Selection.TypeParagraph

    ' Process the Section nodes.
    For Each section_node In article_node.childNodes
        ' Get the Section Title.
        section_title = Trim$(section_node.Attributes(0).nodeValue)

        ' If the Title is non-blank, display it.
        If Len(section_title) > 0 Then
            Selection.Style = ActiveDocument.Styles.Item("Heading 2")
            Selection.TypeText section_title
            Selection.TypeParagraph
        End If

        ' Process the Section's Paragraph nodes.
        For Each paragraph_node In section_node.childNodes
            Selection.Style = ActiveDocument.Styles.Item("Normal")
            Selection.TypeText vbTab & StripWhitespace(paragraph_node.childNodes(0).nodeValue)
            Selection.TypeParagraph
        Next paragraph_node
    Next section_node
End Sub
' Remove leading and trailing invisible characters from the string.
Private Function StripWhitespace(ByVal txt As String) As String
    Dim ch As String
    Dim pos1 As Integer
    Dim pos2 As Integer

    ' Find the first non-whitespace character.
    For pos1 = 1 To Len(txt)
        ch = Mid$(txt, pos1, 1)
        If ch > " " And ch <= "~" Then Exit For
    Next pos1

    ' Find the first non-whitespace character.
    For pos2 = Len(txt) To pos1 Step -1
        ch = Mid$(txt, pos2, 1)
        If ch > " " And ch <= "~" Then Exit For
    Next pos2

    ' Return the text between pos1 and pos2, inclusive.
    If pos1 > pos2 Then
        StripWhitespace = ""
    Else
        StripWhitespace = Mid$(txt, pos1, pos2 - pos1 + 1)
    End If
End Function
Sub TestLoadXmlArticle()
    LoadXmlArticle "C:\VB XML\Src\Ch10\XmlWord\Data\Article.xml"
End Sub
Sub TestSaveXmlArticle()
    SaveXmlArticle "C:\VB XML\Src\Ch10\XmlWord\Data\Article.xml"
End Sub
' Save the currently active document in an XML file
' containing an article.
Sub SaveXmlArticle(ByVal xml_file_name As String)
    Dim word_paragraph As Paragraph
    Dim dom_document As DOMDocument
    Dim processing_instruction As IXMLDOMProcessingInstruction
    Dim article_element As IXMLDOMElement
    Dim section_element As IXMLDOMElement
    Dim paragraph_element As IXMLDOMElement

    ' Create the new DOM document.
    Set dom_document = New DOMDocument
    Set processing_instruction = dom_document.createProcessingInstruction("xml", "version='1.0'")
    dom_document.appendChild processing_instruction

    ' Process the active document.
    For Each word_paragraph In ActiveDocument.Paragraphs
        ' Process this paragraph.
        Select Case word_paragraph.Style
            Case "Heading 1"
                ' Make the Article element.
                Set article_element = MakeArticle( _
                    dom_document, _
                    dom_document, _
                    StripWhitespace(word_paragraph.Range.Text))

            Case "Heading 2"
                ' Make a new Section element.
                Set section_element = MakeSection( _
                    dom_document, _
                    article_element, _
                    StripWhitespace(word_paragraph.Range.Text), _
                    2)

            Case "Normal"
                ' If there's no Section yet, make one with a blank Title.
                If section_element Is Nothing Then
                    Set section_element = MakeSection( _
                        dom_document, _
                        article_element, _
                        StripWhitespace(word_paragraph.Range.Text), _
                        2)
                End If

                ' Make a new Paragraph element.
                Set paragraph_element = MakeParagraph( _
                    dom_document, _
                    section_element, _
                    StripWhitespace(word_paragraph.Range.Text), _
                    4)
        End Select
    Next word_paragraph

    ' Save the DOM document.
    dom_document.Save xml_file_name
    Set dom_document = Nothing
End Sub
' Make an Article with the indicated Title.
' Return the new Article element.
Private Function MakeArticle(ByVal dom_document As DOMDocument, ByVal parent As Object, ByVal title As String) As IXMLDOMElement
    Dim article_element As IXMLDOMElement

    ' Make the Article element.
    Set article_element = dom_document.createElement("Article")
    parent.appendChild article_element
    article_element.setAttribute "Title", title

    ' Start a new line inside the Article element.
    article_element.appendChild dom_document.createTextNode(vbCrLf)

    ' Return the new Article element.
    Set MakeArticle = article_element
End Function
' Make a Section with the indicated Title.
' Return the new Section element.
Private Function MakeSection(ByVal dom_document As DOMDocument, ByVal article_element As IXMLDOMElement, ByVal title As String, ByVal indent_level As Integer) As IXMLDOMElement
    Dim section_element As IXMLDOMElement

    ' Indent.
    article_element.appendChild dom_document.createTextNode(Space$(indent_level))

    ' Make a new Section element.
    Set section_element = dom_document.createElement("Section")
    article_element.appendChild section_element
    section_element.setAttribute "Title", StripWhitespace(title)

    ' Start a new line inside the Section element.
    section_element.appendChild dom_document.createTextNode(vbCrLf & Space$(indent_level))

    ' Add a new line after the </Section>.
    article_element.appendChild dom_document.createTextNode(vbCrLf)

    ' Return the new Section element.
    Set MakeSection = section_element
End Function
' Make a Paragraph with the indicated text.
' Return the new Paragraph element.
Private Function MakeParagraph(ByVal dom_document As DOMDocument, ByVal parent As IXMLDOMElement, ByVal text_value As String, ByVal indent_level As Integer) As IXMLDOMElement
    Dim paragraph_element As IXMLDOMElement

    ' Make sure there's something here.
    If Len(text_value) = 0 Then Exit Function

    ' Indent.
    parent.appendChild dom_document.createTextNode(Space$(2))

    ' Make a new Paragraph element.
    Set paragraph_element = dom_document.createElement("P")
    parent.appendChild paragraph_element

    ' Start a new line inside the Paragraph element.
    paragraph_element.appendChild dom_document.createTextNode(vbCrLf)

    ' Make the paragraph text element.
    MakeParagraphText dom_document, paragraph_element, text_value, indent_level + 2

    ' Indent the </P> tag.
    paragraph_element.appendChild dom_document.createTextNode(Space$(indent_level))

    ' Add a new line after the </P>.
    parent.appendChild dom_document.createTextNode(vbCrLf & Space$(indent_level - 2))

    ' Return the new Paragraph element.
    Set MakeParagraph = paragraph_element
End Function
' Make a new TextNode.
' Return the new node.
Private Function MakeParagraphText(ByVal dom_document As DOMDocument, ByVal parent As IXMLDOMElement, ByVal text_value As String, ByVal indent_level As Integer) As IXMLDOMElement
    Dim text_node As IXMLDOMText

    ' Indent.
    parent.appendChild dom_document.createTextNode(Space$(indent_level))

    ' Create the TextNode.
    Set text_node = dom_document.createTextNode(text_value)
    parent.appendChild text_node

    ' Add a new line after the text.
    parent.appendChild dom_document.createTextNode(vbCrLf)
End Function
' Load the data in the XML document into a Word table
' in the currently active document.
Sub LoadXmlTable(ByVal xml_file_name As String)
Attribute LoadXmlTable.VB_Description = "Macro recorded 8/22/2001 by Rod"
Attribute LoadXmlTable.VB_ProcData.VB_Invoke_Func = "Normal.NewMacros.MakeTable"
    Dim word_table As Table
    Dim dom_document As DOMDocument
    Dim table_node As IXMLDOMNode
    Dim row_node As IXMLDOMNode
    Dim column_node As IXMLDOMNode
    Dim num_rows As Integer
    Dim num_cols As Integer
    Dim r As Integer
    Dim c As Integer

    ' Load the XML file into a DOM representation.
    Set dom_document = New DOMDocument
    dom_document.Load xml_file_name

    ' Get the Books node.
    Set table_node = dom_document.documentElement

    ' See how many rows there are.
    num_rows = table_node.childNodes.Length

    ' See how many columns there are.
    Set row_node = table_node.childNodes(0)
    num_cols = row_node.childNodes.Length

    ' Create the new table.
    Set word_table = ActiveDocument.Tables.Add( _
        Range:=Selection.Range, _
        NumRows:=num_rows, _
        NumColumns:=num_cols, _
        DefaultTableBehavior:=wdWord9TableBehavior, _
        AutoFitBehavior:=wdAutoFitContent)

    ' Enter the data.
    r = 1
    For Each row_node In table_node.childNodes
        c = 1
        For Each column_node In row_node.childNodes
            word_table.Cell(r, c).Range.Text = column_node.Text
            c = c + 1
        Next column_node
        r = r + 1
    Next row_node
End Sub
Sub TestLoadXmlTable()
    LoadXmlTable "C:\VB XML\Src\Ch10\XmlWord\Data\Table.xml"
End Sub
' Save the currently selected table in an XML file.
Sub SaveXmlTable(ByVal xml_file_name As String)
    Dim word_table As Table
    Dim dom_document As DOMDocument
    Dim processing_instruction As IXMLDOMProcessingInstruction
    Dim table_element As IXMLDOMElement
    Dim row_element As IXMLDOMElement
    Dim cell_element As IXMLDOMElement
    Dim r As Integer
    Dim c As Integer

    ' Create the new DOM document.
    Set dom_document = New DOMDocument
    Set processing_instruction = dom_document.createProcessingInstruction("xml", "version='1.0'")
    dom_document.appendChild processing_instruction

    ' Find the table.
    Set word_table = Selection.Tables.Item(1)

    ' Write the Books element.
    Set table_element = dom_document.createElement("Books")
    dom_document.appendChild table_element

    ' Write the rows.
    For r = 1 To word_table.Rows.Count
        ' Indent.
        table_element.appendChild dom_document.createTextNode(vbCrLf & "  ")

        ' Create the Book element.
        Set row_element = dom_document.createElement("Book")
        table_element.appendChild row_element

        ' Write this row's columns.
        For c = 1 To word_table.Columns.Count
            ' Indent.
            row_element.appendChild dom_document.createTextNode(vbCrLf & "    ")

            ' Create the BookInfo element.
            Set cell_element = dom_document.createElement("BookInfo")
            row_element.appendChild cell_element

            ' Add the cell's text.
            cell_element.appendChild dom_document.createTextNode(StripWhitespace(word_table.Cell(r, c).Range.Text))
        Next c

        ' Indent before the </Book> tag.
        row_element.appendChild dom_document.createTextNode(vbCrLf & "  ")
    Next r

    ' New line before the </Books> tag.
    table_element.appendChild dom_document.createTextNode(vbCrLf)

    ' Save the DOM document.
    dom_document.Save xml_file_name
    Set dom_document = Nothing
End Sub
Sub TestSaveXmlTable()
    SaveXmlTable "C:\VB XML\Src\Ch10\XmlWord\Data\Table.xml"
End Sub
